import * as Path from "path";
import { Tree } from "@himenon/path-oriented-data-structure";
import Dot from "dot-prop";
import { UnSupportError } from "../../Exception";
import * as Def from "./Definition";
import * as Operation from "./Operation";
import * as State from "./State";
import * as Structure from "./structure";
export const create = (factory, rootDocument) => {
    const state = State.createDefaultState(rootDocument);
    const { operator, getChildByPaths } = Structure.create();
    const isAfterDefined = (referencePath) => {
        return !!Dot.get(state.document, referencePath.replace(/\//g, "."));
    };
    const convertNamespace = (tree) => {
        const statements = [];
        Object.values(tree.getChildren()).map(child => {
            if (child instanceof Tree || child instanceof Structure.NamespaceTree.Item) {
                statements.push(convertNamespace(child));
            }
            else if (child instanceof Structure.InterfaceNode.Item) {
                statements.push(child.value);
            }
            else if (child instanceof Structure.TypeAliasNode.Item) {
                statements.push(child.value);
            }
        });
        if (tree instanceof Structure.NamespaceTree.Item) {
            return factory.Namespace.create({
                export: true,
                name: tree.params.name,
                statements,
                comment: tree.params.comment,
                deprecated: tree.params.deprecated,
            });
        }
        return factory.Namespace.create({
            export: true,
            name: tree.name,
            statements,
        });
    };
    const getRootStatements = () => {
        // fs.writeFileSync("debug/tree.json", JSON.stringify(operator.getHierarchy(), null, 2), { encoding: "utf-8" });
        const statements = Def.componentNames.reduce((statements, componentName) => {
            const treeOfNamespace = getChildByPaths(componentName, "namespace");
            if (treeOfNamespace) {
                return statements.concat(convertNamespace(treeOfNamespace));
            }
            return statements;
        }, []);
        return statements;
    };
    const getAdditionalStatements = () => {
        return state.additionalStatements;
    };
    return {
        hasStatement: (path, types) => {
            const alreadyRegistered = types.some(type => !!operator.getChildByPaths(path, type));
            return alreadyRegistered;
        },
        addStatement: (path, statement) => {
            if (!path.startsWith("components")) {
                throw new UnSupportError(`componentsから始まっていません。path=${path}`);
            }
            const targetPath = Path.posix.relative("components", path);
            operator.set(targetPath, Structure.createInstance(statement));
        },
        getStatement: (path, kind) => {
            const targetPath = Path.posix.relative("components", path);
            return getChildByPaths(targetPath, kind);
        },
        getRootStatements,
        getAdditionalStatements,
        addComponent: (componentName, statement) => {
            operator.set(`${componentName}`, Structure.createInstance(statement));
        },
        getNoReferenceOperationState: () => Operation.create(state.document),
        updateOperationState: (httpMethod, requestUri, operationId, newOperationState) => {
            let operationState = state.operations[operationId];
            if (operationState) {
                operationState = { ...operationState, ...newOperationState };
            }
            else {
                operationState = State.createDefaultOperationState(httpMethod, requestUri, newOperationState);
            }
            state.operations[operationId] = operationState;
        },
        addAdditionalStatement: (statements) => {
            state.additionalStatements = state.additionalStatements.concat(statements);
        },
        getPathItem: (localPath) => {
            if (!localPath.startsWith("components/pathItem")) {
                throw new Error("Only use start with 'component/pathItems': " + localPath);
            }
            const result = Dot.get(state.document, localPath.replace(/\//g, "."));
            if (!result) {
                throw new Error(`Not found ${localPath}`);
            }
            return result;
        },
        getParameter: (localPath) => {
            if (!localPath.startsWith("components/parameters")) {
                throw new Error("Only use start with 'component/parameters': " + localPath);
            }
            const result = Dot.get(state.document, localPath.replace(/\//g, "."));
            if (!result) {
                throw new Error(`Not found ${localPath}`);
            }
            return result;
        },
        isAfterDefined,
    };
};
//# sourceMappingURL=Store.js.map